declare type FindManyUser_balancesType = {
    data: {
        findManyUser_balances: {
            market_id: number;
            deposits: string;
            brute_deposits: string;
            borrows: string;
            brute_borrows: string;
            markets: {
                market_type: string;
                contract_address: string;
                name: string;
                id: number;
                is_listed: boolean;
            }[];
        }[];
    };
};
declare type FindManyMarketsType = {
    data: {
        findManyMarkets: {
            id: number;
            name: string;
            contract_address: string;
            is_listed: boolean;
            borrow_rate: string;
            total_borrows: string;
            total_supply: string;
            underlying_token_price: string;
            underlying_token_address: string;
            supply_rate: string;
        }[];
    };
};
/**
 * Retrieves the balance of a user.
 * @param user_address The address of the user.
 * @returns A promise that resolves to an object containing the user's balance data, or null if the user is not found. The balance data includes the user's deposits, borrows, and interest for each market.
 */
export declare const getUserBalance: (user_address: string, chainId: number) => Promise<FindManyUser_balancesType | null>;
/**
 * Retrieves a list of markets from the server.
 * @param chainId - The ID of the chain to fetch the markets from.
 * @returns A promise that resolves to the list of markets, or null if an error occurs.
 */
export declare const getMarkets: (chainId: number) => Promise<FindManyMarketsType | null>;
export {};
